#' Calculate transformations of the mean vector.
#' @param xvec A vector with two components, representing values of observation means.
#' @param K A bivariate cumulant generating function
#' @param hattau Bivariate MLE, if given instead of xvec
#' @param verbose Print diagnostic data.
#' @return A list including maximum likelihood estimates, signed roots of the likelihood ratio statistic, reduced maximum likelihood estimates, and the derivative of the slope.
#' @export
#' @importFrom nleqslv nleqslv
hatwvec <- function(xvec=NULL,K,hattau=NULL,verbose=FALSE){
   myK<-K
   if(verbose) cat("Entered hatwvec, xvec",xvec,"\n")
#  cat("Calculate tilde\n")
   if(is.null(hattau)){
      if(verbose) cat("Branch 1\n")
      tildetau <- c(0,tautilde(0,xvec,K=myK))
      hattau <- if(is.na(tildetau[2])) c(NA,NA) else
         try(nleqslv(tildetau,function(tauv,xb) return(K(tauv)$d1-xb),xb=xvec,
            control=list(xtol=1.0e-12))$x)
      if(verbose) cat("Branch 1b, hattau",hattau,"\n")
      if(inherits(hattau,"try-error")) hattau <- c(NA,NA)
      if(verbose) cat("Finished first optimization, hattau",hattau,"\n")
   }else{
      if(verbose) cat("Branch 2\n")
      xvec<-K(hattau)$d1
      tildetau <- c(0,tautilde(0,xvec,K=myK))
      if(verbose) cat("tildetau",tildetau,"\n")
   }
   if(verbose) cat("Calculated hattau\n")
   big <- xvec%*%hattau -K(hattau)$d0 
   if(verbose) cat("big",big,"\n")
   if(is.na(big)) cat("NA in Big",xvec,"hattau",hattau,"\n")
   small <- xvec%*%tildetau -K(tildetau)$d0 
   if(verbose) cat("small",small,"\n")
   hw1 <- sqrt(2*max(big-small,0))
   if(hattau[1]<0) hw1 <- -hw1
   hw2 <- sqrt(2*max(small,0))
   if(tildetau[2]<0) hw2 <- -hw2
   if(verbose) cat("hw1",hw1,"hw2",hw2,"\n")
   hv <- list(what=c(hw1,hw2),hattau=hattau,tildetau=tildetau,big=big,small=small,xvec=xvec)
   if(verbose) cat("In hatwvec tildetau[2]",tildetau[1],"hw1",hw1,"\n")
   hv$slope <- c(((hattau[2]-tildetau[2])*hw2)/( tildetau[2]*hw1),
      -zeta2(NA,xvec,K,hv))
   ht1z <- c(hattau[1],0)
   Kout <- K(ht1z)
   Khatstr <- K(hattau)$d2
   sigma <-sqrt(c(Khatstr[1,1]-Khatstr[1,2]*Khatstr[1,2]/Khatstr[2,2],K(tildetau)$d2[2,2]))
   hv$gv <- c(hv$what[1],hv$what[2]+hv$slope[2]*hv$what[1])
   hv$uv <- c(hv$hattau[1],hv$tildetau[2])*sigma
   w2z <- wvec(ht1z,xvec,hv,K)[1,2]
   if(verbose) cat("-hv$slope[2]",-hv$slope[2],"w2z/hw1",w2z/hw1,"\n")
#  cat("first", (Kout$d1[1]-xvec[1])/((w2z-hw2)*hw1*sigma[1]),"second",w2z/hw1^2,"\n")
#  hv$gp <- (Kout$d1[1]-xvec[1])/((w2z-hw2)*hw1*sigma[1]) - (w2z/hw1)/hw1
   hv$gp <- ((Kout$d1[1]-xvec[1])/((w2z-hw2)*sigma[1]) - (w2z/hw1))/hw1
#  cat("Leaving hatwvec\n")
   return(hv)
}
