#' Calculate the two-dimensional LR saddlepoint approximation.  Also, sketch out the boundary of the region whose probability is calcualted.
#' @param xvec Two-component vector at which to calculate the tail probability.
#' @param npts Number of points at which to calculate each leg of the boundary.
#' @param xscale A factor to scale the distance into the tails for boundary calculation.
#' @param n number of independent bivariate vectors making up the vector mean whose probability is being calculated.
#' @param mcsize Number of Monte Carlo samples to use for approximation of tail.
#' @param rangemult Multiplier controling how far below and to the left of the set vertex to plot
#' @param model a list with two components K, representing the bivariate cumulant generating function of each summand, and sim, representing the code to simulate random vectors.
#' @param plotme Logical variable indicating whether MC samples should be plotted.
#' @param getcor Logical variable indicating SRLLR statistics should be calculated for all values of the Monte Carlo samples.
#' @param delta Parameter needed to test derivatives.
#' @param verbose Logical flag controling diagnostic output.
#' @param correct Logical flag controling whether likelihood ratio statistics are transformed to have a rectangular region 
#' @return A list of points representing the boundaries of the regions.
#' @examples myplot()
#' @export
#' @importFrom stats cor qqnorm var
#' @importFrom graphics lines.default par plot.default points.default segments contour
myplot <- function(xvec=c(4,5),npts=20,xscale=1, n=1,mcsize=10000,rangemult=2,
      model=list(K=Kdep, sim=simgamma),plotme=TRUE,getcor=TRUE,delta=0.01,
      verbose=FALSE,correct=TRUE){
   if(verbose) cat("Entered myplot\n")
   mycgf <- model$K(rep(0,2))
   if(is.null(xvec)) xvec <- mycgf$d1+c(1,1.9)/sqrt(n)
   if(verbose) cat("xvec=",xvec,"\n")
   xxout <- array(NA,c(npts,2,2))
   tautilde <- tauhat <- array(NA,c(npts,2,2))
   boundary <- array(NA,c(npts,2,2))
   zetav <- rep(NA,npts)
   out<-list(NULL,NULL)
   for(j in rev(seq(npts))){
      for(k in 1:2){
         xxout[j,,k] <- xvec
         xxout[j,k,k] <- xvec[k]+3*sqrt(mycgf$d2[k,k])*(j-1)*xscale/npts
         out[[k]] <- hatwvec(xvec=xxout[j,,k],K=model$K)
         tauhat[j,,k] <- out[[k]]$hattau
         tautilde[j,,k] <- out[[k]]$tildetau
         boundary[j,,k] <- out[[k]]$what
      }
      zetav[j] <- zeta2(boundary[j,1,1],xvec,model$K,out[[2]])
   }
#  print(out[[1]])
   Khat<-model$K(out[[1]]$hattau)
#  checkcheck(out[[1]],delta,model,xvec,Khat$d2)
   twodout<-twodsp(xvec,n=n, K=model$K,stats=out[[1]])
   if(correct) boundary[,1,]<-(boundary[,1,]+twodout$zeta2*boundary[,2,])/sqrt(1+twodout$zeta2^2)#check
   if(plotme){
#     par(mfrow=c(3,2))
      cat("At plot\n")
      pr<-list(range(c(boundary[,1,],out[[1]]$what[1]-rangemult)),range(c(boundary[,2,],out[[1]]$what[2]-rangemult)))
#     PictexPlot::plot.default(
      plot.default(
         pr[[1]],pr[[2]],type="n", xlab="w1",ylab="w2",
         sub=paste("Correlated gamma example, xv",paste(xvec,collapse=",")))
      cat("Back in myplot")
#     for(jj in 1:2) PictexPlot::lines.default(boundary[,1,jj],boundary[,2,jj])
      for(jj in 1:2) lines.default(boundary[,1,jj],boundary[,2,jj])
      rr<- par("usr")
   }
   dd<-2.5*sqrt((boundary[npts,2,2]-boundary[1,2,1])^2+(boundary[npts,2,1]-boundary[1,2,1])^2)/2
   theta<- atan(out[[1]]$slope)
   if(verbose) cat("theta=",theta, "out[[1]]$slope=",out[[1]]$slope,"\n")
   if(plotme){
      for(jj in 1:2){
         cat("About to draw segments")
#        PictexPlot::segments(out[[1]]$what[1],out[[1]]$what[2],out[[1]]$what[1]+dd*sin(theta[jj]),
         segments(out[[1]]$what[1],out[[1]]$what[2],out[[1]]$what[1]+dd*sin(theta[jj]),
            out[[1]]$what[2]+dd*cos(theta[jj]),lty=1+jj,col=2+jj)
      }
#    PictexPlot::legend(pr[[1]][1],pr[[2]][2],legend=c("Tangent","Best"),lty=2:3,col=3:4)
     legend(pr[[1]][1],pr[[2]][2],legend=c("Tangent","Best"),lty=2:3,col=3:4)
   }
#  locator(1)
   if(verbose) cat("n=",n,"\n")
#  cat("Bivariate Saddlepoint including curve",twodout,"\n")
   if(mcsize>0){
      xout <- model$sim(mcsize,n)
#     print("xvec");print(xvec);print("xout") ; print(xout)
      stats<-calcsummaries(xout,xvec,model$K,hard=(getcor|plotme))
      cat("mc1",mean(stats$okm[,1]),
         "1-pnorm(out[[1]]$what[1])",1-pnorm(out[[1]]$what[1]),
         "correct",1-pnorm(out[[1]]$what[1])+dnorm(out[[1]]$what[1])*(1/(sqrt(Khat$d2[1,1])*out[[1]]$hattau[1])-1/out[[1]]$what[1]),
        "\n")
      mcout<-mean(stats$ok)
      cat("mc",mcout+c(-1,1)*1.96*sqrt(mcout*(1-mcout)/mcsize),"\n")
      if(correct) stats$wm[,1]<-(stats$wm[,1]+twodout$zeta2*stats$wm[,2])/sqrt(1+twodout$zeta2^2)#check
      if(plotme){
         points.default(stats$wm[,1],stats$wm[,2],col=1+stats$ok)
         menu(1)
         par(mfrow=c(2,2))
         qqnorm(sort(stats$wm[,1]),sub="srllr1",type="l"); abline(0,1)
         qqnorm(sort(stats$wm[,2]),sub="srllr2",type="l"); abline(0,1)
         qqnorm(sort(xout[,1]),sub="x1",type="l"); abline(0,1)
         qqnorm(sort(xout[,2]),sub="x2",type="l"); abline(0,1)
         cat("mean(stats$wm[,1])",mean(stats$wm[,1]),"\n")
         cat("mean(stats$wm[,2])",mean(stats$wm[,2]),"\n")
         cat("var(stats$wm[,1])",var(stats$wm[,1]),"\n")
         cat("var(stats$wm[,2])",var(stats$wm[,2]),"\n")
         menu(1)
         par(mfrow=c(1,2))
         plot.default(xout[,1],stats$wm[,1])
         plot.default(xout[,2],stats$wm[,2])
         par(mfrow=c(1,1))
      }
      cat(var(stats$wm[,2]))
      if(getcor|plotme) cat("W Cor",cor(stats$wm[,1],stats$wm[,2]),"\n")
   }
   ht1z<- c(out[[1]]$hattau[1],0)
   if(verbose) cat("out[[1]]$gp",out[[1]]$gp,"\n")
   if(verbose) cat("d zeta2",( zeta2(out[[1]]$what[1]+delta/2,xvec,model$K,out[[1]])-
      zeta2(out[[1]]$what[1]-delta/2,xvec,model$K,out[[1]]))/delta,"\n")
   return(invisible(list(w1=boundary[,,1],w2=boundary[,,2],
      tautilde1=tautilde[,,1],tautilde2=tautilde[,,2],tauhat1=tauhat[,,1],tauhat2=tauhat[,,2],
      slope=out[[1]]$slope,gp=out[[1]]$gp,xvec=xvec)))
}
