#' Test the two-dimensional LR saddlepoint approximation.
#' @param ll Lower left corner of region to be investigated.
#' @param ur Upper right corner of region to be investigated.
#' @param npts Number of points at which to calculate each leg of the boundary.
#' @param n number of independent bivariate vectors making up the vector mean whose probability is being calculated.
#' @param mcsize Number of Monte Carlo samples to use for approximation of tail.
#' @param model a list with two components K, representing the bivariate cumulant generating function of each summand, and sim, representing the code to simulate random vectors.
#' @param out results of a previous run.
#' @param verbose logical flag passed to twodsp triggering verbose output.
#' @param abserror logical indicating whether error should have absolute value taken.
#' @return A three-dimensional array, with the first two dimensions associated with values for the two ordinates, and the last reflecting the various approximation
#' @importFrom graphics plot points filled.contour
#' @export
#' @examples 
#' test1<-testsp(ll=c(2,2),ur=c(4,4))
#' test2<-testsp()
#' test3<-testsp(model=list(K=Kbvn,sim=simbvn))
#' test4<-testsp(model=list(K=Kindexp,sim=simindexp))
testsp <- function(ll=NULL,ur=NULL,npts=21,n=1,mcsize=1000, model=list(K=Kdep, sim=simgamma),out=NULL,verbose=FALSE,abserror=TRUE){
   if(is.null(out)){
      Kzs<-model$K(c(0,0))
      if(is.null(ll)) ll=Kzs$d1
      if(is.null(ur)) ur=ll+2*sqrt(diag(Kzs$d2))/sqrt(n)
      if(verbose) cat("Entered testsp\n")
      x1<-ll[1]+(0:(npts-1))*(ur[1]-ll[1])/max(npts-1,1)
      x2<-ll[2]+(0:(npts-1))*(ur[2]-ll[2])/max(npts-1,1)
      if(verbose) cat("x1",x1,"x2",x2,"\n")
      for(ii in 0:(npts-1)) for(jj in 0:(npts-1)){
         if(verbose) cat("ii=",ii," jj=",jj,"xvec",c(x1[ii+1],x2[jj+1]),"\n")
         twodout <- twodsp(xvec=c(x1[ii+1],x2[jj+1]),n=n,K=model$K,stats=NULL,verbose=verbose)
         if((ii==0)&&(jj==0)){
            out<-array(NA,c(npts,npts,length(twodout$approx)+1))
            tauout<-array(NA,c(npts,npts,length(twodout$hattau),2))
            dimnames(out)<-list(as.character(x1),as.character(x2),
               c(names(twodout$approx),"mc"))
            xout <- model$sim(mcsize,n)
            stats<-calcsummaries(xout,c(3,4),model$K,TRUE)
         }
         out[ii+1,jj+1,seq(length(twodout$approx))] <- twodout$approx
         out[ii+1,jj+1,length(twodout$approx)+1] <- mean((x1[ii+1]<=xout[,1])&(x2[jj+1]<=xout[,2]))
         tauout[ii+1,jj+1,,1]<-twodout$hattau
         tauout[ii+1,jj+1,,2]<-twodout$hgv
      }
   }else{
      x1<-as.numeric(dimnames(out)[[1]])
      x2<-as.numeric(dimnames(out)[[2]])
   }
   if(verbose) cat("cov(xout[,1],xout[,2])",cov(xout[,1],xout[,2]),"\n")
#  if(verbose) cat("dim(out)",dim(out),"\n")
   error<-out[,,-dim(out)[3],drop=FALSE]
#  if(verbose) cat("dim(error)",dim(error),"\n")
   for(jj in 1:(dim(out)[3]-1)) error[,,jj]<-(out[,,jj]-out[,,"mc"])/out[,,"mc"]
   if(abserror) error<-abs(error)
   zl<-range(error,na.rm=FALSE,finite=TRUE)
   if(verbose) cat("zl",zl,"\n")
   par(mfrow=c(2,2))
   if(npts>1){
      for(jj in 1:(dim(out)[3]-1)){
         contour(x1,x2,error[,,jj], main="Error",sub=dimnames(error)[[3]][jj])
         contour(x1,x2,tauout[,,1,1], add=TRUE,col=2,levels=0)
         contour(x1,x2,tauout[,,2,1], add=TRUE,col=3,levels=0)
         contour(x1,x2,tauout[,,1,2], add=TRUE,col=4,levels=0)
         contour(x1,x2,tauout[,,2,2], add=TRUE,col=5,levels=0)
      }
      for(jj in 1:(dim(out)[3]-1)){
         pdf(file=paste("panel",jj,".pdf",sep=""),paper="USr")
         filled.contour(x1,x2,error[,,jj],zlim=zl,sub=dimnames(error)[[3]][jj])
         dev.off()
      }
      system("pdfjam --nup 2x2 --outfile temp.pdf panel1.pdf panel2.pdf panel3.pdf panel4.pdf")
   }
   cat("names(stats)",names(stats),"\n")
   return(list(approx=out,error=error,tauout=tauout,gv=stats$gv,uv=stats$uv,
      tt=stats$tt,ta=stats$ta,wm=stats$wm))
}
